@@ -119,11 +119,12 @@
 #endif
 typedef struct _GraphemeInfo
 {
-  size_t
+  ssize_t
     index,
     x_offset,
     x_advance,
-    y_offset;
+    y_offset,
+    y_advance;
 
   size_t
     cluster;
@@ -486,7 +487,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
       default:
         break;
     }
-    if (draw_info->undercolor.alpha != TransparentAlpha)
+    if (draw_info->undercolor.alpha != (MagickRealType) TransparentAlpha)
       {
         DrawInfo
           *undercolor_info;
@@ -508,7 +509,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
     annotate_info->affine.tx=offset.x;
     annotate_info->affine.ty=offset.y;
     pixel=annotate_info->fill;
-    if (annotate_info->stroke.alpha != TransparentAlpha)
+    if (annotate_info->stroke.alpha != (MagickRealType) TransparentAlpha)
       pixel=annotate_info->stroke;
     (void) QueryColorname(image,&pixel,AllCompliance,color,exception);
     (void) FormatLocaleString(primitive,MagickPathExtent,"stroke %s "
@@ -611,18 +612,23 @@ static inline char *ReplaceSpaceWithNewline(char **caption,char *space)
         *target;
 
       size_t
-        length,
+        length;
+
+      ssize_t
         offset;
 
       length=strlen(*caption);
       *space='\n';
       offset=space-(*caption);
-      target=AcquireString(*caption);
-      CopyMagickString(target,*caption,offset+2);
-      ConcatenateMagickString(target,space+octets,length);
-      (void) DestroyString(*caption);
-      *caption=target;
-      space=(*caption)+offset;
+      if (offset >= 0)
+        {
+          target=AcquireString(*caption);
+          CopyMagickString(target,*caption,(size_t) offset+2);
+          ConcatenateMagickString(target,space+octets,length);
+          (void) DestroyString(*caption);
+          *caption=target;
+          space=(*caption)+offset;
+        }
     }
   return(space);
 }
@@ -692,7 +698,7 @@ MagickExport ssize_t FormatMagickCaption(Image *image,DrawInfo *draw_info,
                 *target;
 
               target=AcquireString(*caption);
-              CopyMagickString(target,*caption,n+1);
+              CopyMagickString(target,*caption,(size_t) n+1);
               ConcatenateMagickString(target,"\n",strlen(*caption)+1);
               ConcatenateMagickString(target,p,strlen(*caption)+2);
               (void) DestroyString(*caption);
@@ -1155,12 +1161,12 @@ static size_t ComplexRaqmTextLayout(const Image *image,
   raqm_glyph_t
     *glyphs;
 
-  ssize_t
-    i;
-
   size_t
     extent;
 
+  ssize_t
+    i;
+
   extent=0;
   rq=raqm_create();
   if (rq == (raqm_t *) NULL)
@@ -1220,6 +1226,7 @@ static size_t ComplexRaqmTextLayout(const Image *image,
     (*grapheme)[i].x_offset=glyphs[i].x_offset;
     (*grapheme)[i].x_advance=glyphs[i].x_advance;
     (*grapheme)[i].y_offset=glyphs[i].y_offset;
+    (*grapheme)[i].y_advance=glyphs[i].y_advance;
     (*grapheme)[i].cluster=glyphs[i].cluster;
   }
 
@@ -1250,7 +1257,8 @@ static size_t ComplexTextLayout(const DrawInfo *draw_info,const char *text,
   p=text;
   for (i=0; GetUTFCode(p) != 0; p+=GetUTFOctets(p), i++)
   {
-    (*grapheme)[i].index=(ssize_t) FT_Get_Char_Index(face,GetUTFCode(p));
+    (*grapheme)[i].index=(ssize_t) FT_Get_Char_Index(face,(FT_ULong)
+      GetUTFCode(p));
     (*grapheme)[i].x_offset=0;
     (*grapheme)[i].y_offset=0;
     if (((*grapheme)[i].index != 0) && (last_glyph != 0))
@@ -1272,7 +1280,8 @@ static size_t ComplexTextLayout(const DrawInfo *draw_info,const char *text,
       }
     (void) FT_Load_Glyph(face,(FT_UInt) (*grapheme)[i].index,flags);
     (*grapheme)[i].x_advance=face->glyph->advance.x;
-    (*grapheme)[i].cluster=p-text;
+    (*grapheme)[i].y_advance=face->glyph->advance.y;
+    (*grapheme)[i].cluster=(size_t) (p-text);
     last_glyph=(*grapheme)[i].index;
   }
   return((size_t) i);
@@ -1399,19 +1408,23 @@ static inline const char *FreetypeErrorMessage(
 static void *FreetypeAlloc(FT_Memory magick_unused(memory),long size)
 {
   magick_unreferenced(memory);
+  if (size < 0)
+    return((void *) NULL);
   if ((size_t) size > GetMaxMemoryRequest())
     return((void *) NULL);
-  return(AcquireMagickMemory(size));
+  return(AcquireMagickMemory((size_t) size));
 }
 
 static void *FreetypeRealloc(FT_Memory magick_unused(memory),
-  long magick_unused(cur_size),long new_size,void *block)
+  long magick_unused(cur_size),long size,void *block)
 {
   magick_unreferenced(memory);
   magick_unreferenced(cur_size);
-  if ((size_t) new_size > GetMaxMemoryRequest())
+  if (size < 0)
     return((void *) NULL);
-  return(ResizeMagickMemory(block,new_size));
+  if ((size_t) size > GetMaxMemoryRequest())
+    return((void *) NULL);
+  return(ResizeMagickMemory(block,(size_t) size));
 }
 
 static void FreetypeFree(FT_Memory magick_unused(memory),void *block)
@@ -1625,8 +1638,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   */
   stream=(FT_StreamRec *) AcquireCriticalMemory(sizeof(*stream));
   (void) memset(stream,0,sizeof(*stream));
-  (void) stat(args.pathname,&attributes);
-  stream->size=attributes.st_size;
+  if (stat(args.pathname,&attributes) == 0)
+    stream->size=attributes.st_size >= 0 ? (unsigned long)
+      attributes.st_size : 0;
   stream->descriptor.pointer=fopen_utf8(args.pathname,"rb");
   stream->read=(&FreetypeReadStream);
   stream->close=(&FreetypeCloseStream);
@@ -1883,7 +1897,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       metrics->bounds.x2=(double) bounds.xMax;
     if ((bounds.yMax > metrics->bounds.y2) && (bounds.yMax != 0))
       metrics->bounds.y2=(double) bounds.yMax;
-    if (((draw_info->stroke.alpha != TransparentAlpha) ||
+    if (((draw_info->stroke.alpha != (MagickRealType) TransparentAlpha) ||
          (draw_info->stroke_pattern != (Image *) NULL)) &&
         ((status != MagickFalse) && (draw_info->render != MagickFalse)))
       {
@@ -1921,9 +1935,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
         /*
           Rasterize the glyph.
         */
-        transparent_fill=((draw_info->fill.alpha == TransparentAlpha) &&
+        transparent_fill=((draw_info->fill.alpha == (MagickRealType) TransparentAlpha) &&
           (draw_info->fill_pattern == (Image *) NULL) &&
-          (draw_info->stroke.alpha == TransparentAlpha) &&
+          (draw_info->stroke.alpha == (MagickRealType) TransparentAlpha) &&
           (draw_info->stroke_pattern == (Image *) NULL)) ? MagickTrue :
           MagickFalse;
         image_view=AcquireAuthenticCacheView(image,exception);
@@ -2007,9 +2021,10 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
                   Sa,
                   Da;
 
-                Da=1.0-(QuantumScale*GetPixelAlpha(image,q));
+                Da=1.0-(QuantumScale*(double) GetPixelAlpha(image,q));
                 Sa=fill_opacity;
-                fill_opacity=(1.0-RoundToUnity(Sa+Da-Sa*Da))*QuantumRange;
+                fill_opacity=(1.0-RoundToUnity(Sa+Da-Sa*Da))*(double)
+                  QuantumRange;
                 SetPixelAlpha(image,fill_opacity,q);
               }
             if (active == MagickFalse)
@@ -2025,7 +2040,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
             status=MagickFalse;
         }
         image_view=DestroyCacheView(image_view);
-        if (((draw_info->stroke.alpha != TransparentAlpha) ||
+        if (((draw_info->stroke.alpha != (MagickRealType) TransparentAlpha) ||
              (draw_info->stroke_pattern != (Image *) NULL)) &&
             (status != MagickFalse))
           {
@@ -2050,6 +2065,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
         origin.x+=MagickMax((FT_Pos) grapheme[i].x_advance,bounds.xMax);
       else
         origin.x+=(FT_Pos) grapheme[i].x_advance;
+    origin.y+=(FT_Pos) grapheme[i].y_advance;
     metrics->origin.x=(double) origin.x;
     metrics->origin.y=(double) origin.y;
     if (metrics->origin.x > metrics->width)
@@ -2349,7 +2365,7 @@ static MagickBooleanType RenderPostscript(Image *image,
       annotate_image=DestroyImage(annotate_image);
       return(MagickTrue);
     }
-  if (draw_info->fill.alpha != TransparentAlpha)
+  if (draw_info->fill.alpha != (MagickRealType) TransparentAlpha)
     {
       CacheView
         *annotate_view;
