@@ -276,9 +276,10 @@ MagickExport RandomInfo *AcquireRandomInfo(void)
 */
 MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
 {
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(random_info != (RandomInfo *) NULL);
   assert(random_info->signature == MagickCoreSignature);
+  if (IsEventLogging() != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   LockSemaphoreInfo(random_info->semaphore);
   if (random_info->reservoir != (StringInfo *) NULL)
     random_info->reservoir=DestroyStringInfo(random_info->reservoir);
@@ -330,7 +331,7 @@ static ssize_t ReadRandom(int file,unsigned char *source,size_t length)
     count;
 
   offset=0;
-  for (q=source; length != 0; length-=count)
+  for (q=source; length != 0; length-=(size_t) count)
   {
     count=(ssize_t) read(file,q,length);
     if (count <= 0)
@@ -358,13 +359,13 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
     *chaos,
     *entropy;
 
-  size_t
-    nanoseconds,
-    seconds;
-
   ssize_t
     pid;
 
+  time_t
+    nanoseconds,
+    seconds;
+
   /*
     Initialize random reservoir.
   */
@@ -429,8 +430,8 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
 
     if (gettimeofday(&timer,(struct timezone *) NULL) == 0)
       {
-        seconds=(size_t) timer.tv_sec;
-        nanoseconds=(size_t) (1000UL*timer.tv_usec);
+        seconds=timer.tv_sec;
+        nanoseconds=1000*timer.tv_usec;
       }
   }
 #endif
@@ -477,9 +478,9 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
       nt_nanoseconds;
 
     /*
-      Not crytographically strong but better than nothing.
+      Not cryptographically strong but better than nothing.
     */
-    nt_seconds=NTElapsedTime()+NTUserTime();
+    nt_seconds=NTElapsedTime()+NTElapsedTime();
     SetStringInfoLength(chaos,sizeof(nt_seconds));
     SetStringInfoDatum(chaos,(unsigned char *) &nt_seconds);
     ConcatenateStringInfo(entropy,chaos);
@@ -511,7 +512,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
       *device;
 
     /*
-      Not crytographically strong but better than nothing.
+      Not cryptographically strong but better than nothing.
     */
     if (environ != (char **) NULL)
       {
@@ -519,7 +520,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
           i;
 
         /*
-          Squeeze some entropy from the sometimes unpredicatble environment.
+          Squeeze some entropy from the sometimes unpredictable environment.
         */
         for (i=0; environ[i] != (char *) NULL; i++)
         {
@@ -719,7 +720,7 @@ MagickExport StringInfo *GetRandomKey(RandomInfo *random_info,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetRandomSecretKey() returns the random secet key.
+%  GetRandomSecretKey() returns the random secret key.
 %
 %  The format of the GetRandomSecretKey method is:
 %
