@@ -75,6 +75,9 @@
 #define BI_JPEG  4
 #undef BI_PNG
 #define BI_PNG  5
+#ifndef BI_ALPHABITFIELDS
+ #define BI_ALPHABITFIELDS 6
+#endif
 #if !defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(__MINGW32__)
 #undef BI_RGB
 #define BI_RGB  0
@@ -278,7 +281,7 @@ static MagickBooleanType DecodeImage(Image *image,const size_t compression,
             */
             x=0;
             y++;
-            p=pixels+y*image->columns;
+            p=pixels+y*(ssize_t) image->columns;
             break;
           }
           case 0x02:
@@ -294,7 +297,7 @@ static MagickBooleanType DecodeImage(Image *image,const size_t compression,
             if (byte == EOF)
               return(MagickFalse);
             y+=byte;
-            p=pixels+y*image->columns+x;
+            p=pixels+y*(ssize_t) image->columns+x;
             break;
           }
           default:
@@ -515,6 +518,63 @@ static MagickBooleanType IsBMP(const unsigned char *magick,const size_t length)
 %
 */
 
+static Image *ReadEmbedImage(const ImageInfo *image_info,Image *image,
+  const char *magick,ExceptionInfo *exception)
+{
+  const void
+    *stream;
+
+  Image
+    *embed_image;
+
+  ImageInfo
+    *embed_info;
+
+  MemoryInfo
+    *pixel_info;
+
+  size_t
+    length;
+
+  ssize_t
+    count;
+
+  unsigned char
+    *pixels;
+
+  /*
+    Read embedded image.
+  */
+  length=(size_t) ((MagickOffsetType) GetBlobSize(image)-TellBlob(image));
+  pixel_info=AcquireVirtualMemory(length,sizeof(*pixels));
+  if (pixel_info == (MemoryInfo *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  pixels=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
+  stream=ReadBlobStream(image,length,pixels,&count);
+  if (count != (ssize_t) length)
+    {
+      pixel_info=RelinquishVirtualMemory(pixel_info);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
+  embed_info=AcquireImageInfo();
+  (void) FormatLocaleString(embed_info->filename,MagickPathExtent,
+    "%s:%s",magick,image_info->filename);
+  embed_image=BlobToImage(embed_info,stream,(size_t) count,exception);
+  embed_info=DestroyImageInfo(embed_info);
+  pixel_info=RelinquishVirtualMemory(pixel_info);
+  (void) CloseBlob(image);
+  if (embed_image != (Image *) NULL)
+    {
+      (void) CopyMagickString(embed_image->filename,image->filename,
+        MagickPathExtent);
+      (void) CopyMagickString(embed_image->magick_filename,
+        image->magick_filename,MagickPathExtent);
+      (void) CopyMagickString(embed_image->magick,image->magick,
+        MagickPathExtent);
+    }
+  return(embed_image);
+}
+
 static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   BMPInfo
@@ -627,6 +687,12 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (image->debug != MagickFalse)
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "  BMP header size: %u",bmp_info.size);
+    if (LocaleNCompare((char *) magick,"CI",2) == 0)
+      {
+        if ((bmp_info.size != 12) && (bmp_info.size != 40) &&
+            (bmp_info.size != 64))
+          ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      }
     if (bmp_info.size > 124)
       ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     if (bmp_info.offset_bits < bmp_info.size)
@@ -717,6 +783,12 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   "  Compression: BI_BITFIELDS");
                 break;
               }
+              case BI_ALPHABITFIELDS:
+              {
+                (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                  "  Compression: BI_ALPHABITFIELDS");
+                break;
+              }
               case BI_PNG:
               {
                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -738,11 +810,15 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
               "  Number of colors: %u",bmp_info.number_colors);
           }
-        if ((bmp_info.size > 40) || (bmp_info.compression == BI_BITFIELDS))
+        if ((bmp_info.size > 40) || (bmp_info.compression == BI_BITFIELDS) ||
+            (bmp_info.compression == BI_ALPHABITFIELDS))
+
           {
             bmp_info.red_mask=ReadBlobLSBLong(image);
             bmp_info.green_mask=ReadBlobLSBLong(image);
             bmp_info.blue_mask=ReadBlobLSBLong(image);
+            if (bmp_info.compression == BI_ALPHABITFIELDS)
+              bmp_info.alpha_mask=ReadBlobLSBLong(image);
           }
         if (bmp_info.size > 40)
           {
@@ -843,8 +919,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               }
             }
-            profile_data=(MagickOffsetType)ReadBlobLSBLong(image);
-            profile_size=(MagickOffsetType)ReadBlobLSBLong(image);
+            profile_data=(MagickOffsetType) ReadBlobLSBLong(image);
+            profile_size=(MagickOffsetType) ReadBlobLSBLong(image);
             (void) ReadBlobLSBLong(image);  /* Reserved byte */
           }
       }
@@ -863,6 +939,26 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
     if (bmp_info.height == 0)
       ThrowReaderException(CorruptImageError,"NegativeOrZeroImageSize");
+    if (bmp_info.compression == BI_JPEG)
+      {
+        /*
+          Read embedded JPEG image.
+        */
+        Image *embed_image = ReadEmbedImage(image_info,image,"jpeg",exception);
+        (void) CloseBlob(image);
+        image=DestroyImageList(image);
+        return(embed_image);
+      }
+    if (bmp_info.compression == BI_PNG)
+      {
+        /*
+          Read embedded PNG image.
+        */
+        Image *embed_image = ReadEmbedImage(image_info,image,"png",exception);
+        (void) CloseBlob(image);
+        image=DestroyImageList(image);
+        return(embed_image);
+      }
     if (bmp_info.planes != 1)
       ThrowReaderException(CorruptImageError,"StaticPlanesValueNotEqualToOne");
     if ((bmp_info.bits_per_pixel != 1) && (bmp_info.bits_per_pixel != 4) &&
@@ -889,6 +985,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         break;
       case BI_BITFIELDS:
         break;
+      case BI_ALPHABITFIELDS:
+        break;
       case BI_JPEG:
         ThrowReaderException(CoderError,"JPEGCompressNotSupported");
       case BI_PNG:
@@ -986,7 +1084,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((MagickSizeType) (length/256) > blob_size)
       ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
     if ((bmp_info.compression == BI_RGB) ||
-        (bmp_info.compression == BI_BITFIELDS))
+        (bmp_info.compression == BI_BITFIELDS) ||
+        (bmp_info.compression == BI_ALPHABITFIELDS))
       {
         pixel_info=AcquireVirtualMemory(image->rows,
           MagickMax(bytes_per_line,image->columns+256UL)*sizeof(*pixels));
@@ -1041,7 +1140,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             bytes_per_line=4*(image->columns);
             for (y=(ssize_t) image->rows-1; y >= 0; y--)
             {
-              p=pixels+(image->rows-y-1)*bytes_per_line;
+              p=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
                 if (*(p+3) != 0)
@@ -1149,7 +1248,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (y=(ssize_t) image->rows-1; y >= 0; y--)
         {
-          p=pixels+(image->rows-y-1)*bytes_per_line;
+          p=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (Quantum *) NULL)
             break;
@@ -1177,8 +1276,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
-                (image->rows-y),image->rows);
+              status=SetImageProgress(image,LoadImageTag,((MagickOffsetType)
+                image->rows-y),image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -1193,7 +1292,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (y=(ssize_t) image->rows-1; y >= 0; y--)
         {
-          p=pixels+(image->rows-y-1)*bytes_per_line;
+          p=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (Quantum *) NULL)
             break;
@@ -1223,8 +1322,8 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             break;
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
-                (image->rows-y),image->rows);
+              status=SetImageProgress(image,LoadImageTag,((MagickOffsetType)
+                image->rows-y),image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -1242,7 +1341,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           bytes_per_line=image->columns;
         for (y=(ssize_t) image->rows-1; y >= 0; y--)
         {
-          p=pixels+(image->rows-y-1)*bytes_per_line;
+          p=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (Quantum *) NULL)
             break;
@@ -1254,11 +1353,11 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
-          offset=(MagickOffsetType) (image->rows-y-1);
+          offset=((MagickOffsetType) image->rows-y-1);
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
-                (image->rows-y),image->rows);
+              status=SetImageProgress(image,LoadImageTag,((MagickOffsetType)
+                image->rows-y),image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -1286,14 +1385,14 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         image->storage_class=DirectClass;
         for (y=(ssize_t) image->rows-1; y >= 0; y--)
         {
-          p=pixels+(image->rows-y-1)*bytes_per_line;
+          p=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (Quantum *) NULL)
             break;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
             pixel=(unsigned int) (*p++);
-            pixel|=(*p++) << 8;
+            pixel|=(unsigned int) (*p++) << 8;
             red=((pixel & bmp_info.red_mask) << shift.red) >> 16;
             if (quantum_bits.red == 5)
               red|=((red & 0xe000) >> 5);
@@ -1327,11 +1426,11 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
-          offset=(MagickOffsetType) (image->rows-y-1);
+          offset=((MagickOffsetType) image->rows-y-1);
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
-                (image->rows-y),image->rows);
+              status=SetImageProgress(image,LoadImageTag,((MagickOffsetType)
+                image->rows-y),image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -1346,7 +1445,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
         bytes_per_line=4*((image->columns*24+31)/32);
         for (y=(ssize_t) image->rows-1; y >= 0; y--)
         {
-          p=pixels+(image->rows-y-1)*bytes_per_line;
+          p=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (Quantum *) NULL)
             break;
@@ -1360,11 +1459,11 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
-          offset=(MagickOffsetType) (image->rows-y-1);
+          offset=((MagickOffsetType) image->rows-y-1);
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
-                (image->rows-y),image->rows);
+              status=SetImageProgress(image,LoadImageTag,((MagickOffsetType)
+                image->rows-y),image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -1390,7 +1489,7 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
             alpha,
             pixel;
 
-          p=pixels+(image->rows-y-1)*bytes_per_line;
+          p=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (Quantum *) NULL)
             break;
@@ -1425,11 +1524,11 @@ static Image *ReadBMPImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
-          offset=(MagickOffsetType) (image->rows-y-1);
+          offset=((MagickOffsetType) image->rows-y-1);
           if (image->previous == (Image *) NULL)
             {
-              status=SetImageProgress(image,LoadImageTag,(MagickOffsetType)
-                (image->rows-y),image->rows);
+              status=SetImageProgress(image,LoadImageTag,((MagickOffsetType)
+                image->rows-y),image->rows);
               if (status == MagickFalse)
                 break;
             }
@@ -1886,8 +1985,8 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
               bmp_info.size=124;
               extra_size+=16;
             }
-          bmp_info.file_size+=extra_size;
-          bmp_info.offset_bits+=extra_size;
+          bmp_info.file_size+=(unsigned int) extra_size;
+          bmp_info.offset_bits+=(unsigned int) extra_size;
         }
     if (((ssize_t) image->columns != (ssize_t) ((signed int) image->columns)) ||
         ((ssize_t) image->rows != (ssize_t) ((signed int) image->rows)))
@@ -1944,13 +2043,13 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           p=GetVirtualPixels(image,0,y,image->columns,1,exception);
           if (p == (const Quantum *) NULL)
             break;
-          q=pixels+(image->rows-y-1)*bytes_per_line;
+          q=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           bit=0;
           byte=0;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
             byte<<=1;
-            byte|=GetPixelIndex(image,p) != 0 ? 0x01 : 0x00;
+            byte|=(size_t) (GetPixelIndex(image,p) != 0 ? 0x01 : 0x00);
             bit++;
             if (bit == 8)
               {
@@ -1995,7 +2094,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           p=GetVirtualPixels(image,0,y,image->columns,1,exception);
           if (p == (const Quantum *) NULL)
             break;
-          q=pixels+(image->rows-y-1)*bytes_per_line;
+          q=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           nibble=0;
           byte=0;
           for (x=0; x < (ssize_t) image->columns; x++)
@@ -2039,7 +2138,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           p=GetVirtualPixels(image,0,y,image->columns,1,exception);
           if (p == (const Quantum *) NULL)
             break;
-          q=pixels+(image->rows-y-1)*bytes_per_line;
+          q=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
             *q++=(unsigned char) ((ssize_t) GetPixelIndex(image,p));
@@ -2067,7 +2166,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           p=GetVirtualPixels(image,0,y,image->columns,1,exception);
           if (p == (const Quantum *) NULL)
             break;
-          q=pixels+(image->rows-y-1)*bytes_per_line;
+          q=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
             unsigned short
@@ -2132,7 +2231,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           p=GetVirtualPixels(image,0,y,image->columns,1,exception);
           if (p == (const Quantum *) NULL)
             break;
-          q=pixels+(image->rows-y-1)*bytes_per_line;
+          q=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
             *q++=ScaleQuantumToChar(GetPixelBlue(image,p));
@@ -2162,7 +2261,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           p=GetVirtualPixels(image,0,y,image->columns,1,exception);
           if (p == (const Quantum *) NULL)
             break;
-          q=pixels+(image->rows-y-1)*bytes_per_line;
+          q=pixels+((ssize_t) image->rows-y-1)*(ssize_t) bytes_per_line;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
             Quantum alpha=GetPixelAlpha(image,p);
@@ -2261,6 +2360,12 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
                "   Compression=BI_BITFIELDS");
              break;
            }
+           case BI_ALPHABITFIELDS:
+           {
+             (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+               "   Compression=BI_BITFIELDS");
+             break;
+           }
            default:
            {
              (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -2332,8 +2437,8 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           (void) WriteBlobLSBLong(image,0x4D424544U);  /* PROFILE_EMBEDDED */
         else
           (void) WriteBlobLSBLong(image,0x73524742U);  /* sRGB */
-        
-        // bounds check, assign .0 if invalid value
+
+        /* bounds check, assign .0 if invalid value */
         if (isgreater(image->chromaticity.red_primary.x, 1.0) ||
             !isgreater(image->chromaticity.red_primary.x, 0.0))
           image->chromaticity.red_primary.x = 0.0;
@@ -2367,21 +2472,21 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
         (void) WriteBlobLSBLong(image,(unsigned int)
           (image->chromaticity.red_primary.y*0x40000000));
         (void) WriteBlobLSBLong(image,(unsigned int)
-          ((1.000f-(image->chromaticity.red_primary.x+
+          ((1.000-(image->chromaticity.red_primary.x+
           image->chromaticity.red_primary.y))*0x40000000));
         (void) WriteBlobLSBLong(image,(unsigned int)
           (image->chromaticity.green_primary.x*0x40000000));
         (void) WriteBlobLSBLong(image,(unsigned int)
           (image->chromaticity.green_primary.y*0x40000000));
         (void) WriteBlobLSBLong(image,(unsigned int)
-          ((1.000f-(image->chromaticity.green_primary.x+
+          ((1.000-(image->chromaticity.green_primary.x+
           image->chromaticity.green_primary.y))*0x40000000));
         (void) WriteBlobLSBLong(image,(unsigned int)
           (image->chromaticity.blue_primary.x*0x40000000));
         (void) WriteBlobLSBLong(image,(unsigned int)
           (image->chromaticity.blue_primary.y*0x40000000));
         (void) WriteBlobLSBLong(image,(unsigned int)
-          ((1.000f-(image->chromaticity.blue_primary.x+
+          ((1.000-(image->chromaticity.blue_primary.x+
           image->chromaticity.blue_primary.y))*0x40000000));
         (void) WriteBlobLSBLong(image,(unsigned int)
           (bmp_info.gamma_scale.x*0x10000));
@@ -2441,15 +2546,15 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
         if (image->debug != MagickFalse)
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
             "  Colormap: %.20g entries",(double) image->colors);
-        bmp_colormap=(unsigned char *) AcquireQuantumMemory((size_t) (1UL <<
-          bmp_info.bits_per_pixel),4*sizeof(*bmp_colormap));
+        bmp_colormap=(unsigned char *) AcquireQuantumMemory((size_t) 1UL <<
+          bmp_info.bits_per_pixel,4*sizeof(*bmp_colormap));
         if (bmp_colormap == (unsigned char *) NULL)
           {
             pixel_info=RelinquishVirtualMemory(pixel_info);
             ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
           }
         q=bmp_colormap;
-        for (i=0; i < (ssize_t) MagickMin((ssize_t) image->colors,(ssize_t) bmp_info.number_colors); i++)
+        for (i=0; i < MagickMin((ssize_t) image->colors,(ssize_t) bmp_info.number_colors); i++)
         {
           *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].blue));
           *q++=ScaleQuantumToChar(ClampToQuantum(image->colormap[i].green));
@@ -2457,7 +2562,7 @@ static MagickBooleanType WriteBMPImage(const ImageInfo *image_info,Image *image,
           if (type > 2)
             *q++=(unsigned char) 0x0;
         }
-        for ( ; i < (ssize_t) (1UL << bmp_info.bits_per_pixel); i++)
+        for ( ; i < (ssize_t) 1UL << bmp_info.bits_per_pixel; i++)
         {
           *q++=(unsigned char) 0x00;
           *q++=(unsigned char) 0x00;
