@@ -53,12 +53,12 @@
 #include "MagickCore/log.h"
 #include "MagickCore/magick.h"
 #include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
 #include "MagickCore/option.h"
 #include "MagickCore/resource_.h"
 #include "MagickCore/quantum-private.h"
 #include "MagickCore/static.h"
 #include "MagickCore/string_.h"
-#include "MagickCore/module.h"
 #include "MagickCore/transform.h"
 #include "MagickCore/utility.h"
 #include "MagickCore/utility-private.h"
@@ -66,8 +66,8 @@
 /*
   Global declarations.
 */
-static const char*
-  intermediate_formats[] =
+static const char
+  *intermediate_formats[] =
   {
     "pam",
     "webp"
@@ -136,9 +136,9 @@ static MagickBooleanType IsVIDEO(const unsigned char *magick,
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  ReadVIDEOImage() reads an binary file in the VIDEO video stream format
-%  and returns it.  It allocates the memory necessary for the new Image
-%  structure and returns a pointer to the new image.
+%  ReadVIDEOImage() reads an binary file in the VIDEO video stream format and
+%  returns it.  It allocates the memory necessary for the new Image structure
+%  and returns a pointer to the new image.
 %
 %  The format of the ReadVIDEOImage method is:
 %
@@ -214,9 +214,7 @@ static Image *ReadVIDEOImage(const ImageInfo *image_info,
     {
       char
         command[MagickPathExtent],
-        message[MagickPathExtent];
-
-      char
+        message[MagickPathExtent],
         *options;
 
       const char
@@ -249,8 +247,8 @@ static Image *ReadVIDEOImage(const ImageInfo *image_info,
           (void) ConcatenateMagickString(options," -pix_fmt rgba",
             MagickPathExtent);
       intermediate_format=GetIntermediateFormat(image_info);
-      (void) FormatLocaleString(command,MagickPathExtent," -vcodec %s",
-        intermediate_format);
+      (void) FormatLocaleString(command,MagickPathExtent,
+        " -vcodec %s -lossless 1",intermediate_format);
       (void) ConcatenateMagickString(options,command,MagickPathExtent);
       AcquireUniqueFilename(read_info->unique);
       (void) AcquireUniqueSymbolicLink(image_info->filename,
@@ -469,9 +467,7 @@ static MagickBooleanType CopyDelegateFile(const char *source,
     source_file;
 
   size_t
-    i;
-
-  size_t
+    i,
     length,
     quantum;
 
@@ -514,7 +510,7 @@ static MagickBooleanType CopyDelegateFile(const char *source,
       return(MagickFalse);
     }
   length=0;
-  for (i=0; ; i+=count)
+  for (i=0; ; i+=(size_t) count)
   {
     count=(ssize_t) read(source_file,buffer,quantum);
     if (count <= 0)
@@ -548,7 +544,8 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
     delay;
 
   Image
-    *clone_images;
+    *clone_images,
+    *p;
 
   ImageInfo
     *write_info;
@@ -559,17 +556,14 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  Image
-    *p;
-
-  ssize_t
-    i;
-
   size_t
     count,
     length,
     scene;
 
+  ssize_t
+    i;
+
   unsigned char
     *blob;
 
@@ -677,9 +671,7 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
     {
       char
         command[MagickPathExtent],
-        message[MagickPathExtent];
-
-      char
+        message[MagickPathExtent],
         *options;
 
       const char
@@ -713,11 +705,10 @@ static MagickBooleanType WriteVIDEOImage(const ImageInfo *image_info,
           status=CopyDelegateFile(filename,image->filename);
           (void) RelinquishUniqueFileResource(filename);
         }
-      else if (*message != '\0')
-        {
+      else
+        if (*message != '\0')
           (void) ThrowMagickException(exception,GetMagickModule(),
             DelegateError,"VideoDelegateFailed","`%s'",message);
-        }
       (void) RelinquishUniqueFileResource(write_info->unique);
   }
   write_info=DestroyImageInfo(write_info);
